require "Window"
require "GameLib"
require "Unit"


local WhereAmI = {} 

local tDefaultSettings = {
	tMainPosition = {nil, nil},
	nMainOpacity = 1,
	bShowHeight = false,
	nTextAlign = 2,
}
local arTextAligns = {
	"left",
	"center",
	"right",
}


function WhereAmI:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	self.bCharacterCreated = false
	self.bIsMoving = false

	self.settings = {}

	for k, v in pairs(tDefaultSettings) do
		self.settings[k] = v
	end

	return o
end

function WhereAmI:Init()
	Apollo.RegisterAddon(self)
end

function WhereAmI:OnLoad()
	Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)

	if GameLib.GetPlayerUnit() then
		self:OnCharacterCreated()
	end    
end

function WhereAmI:OnCharacterCreated()
	if self.bCharacterCreated then
		return
	end

	self.bCharacterCreated = true

	Apollo.RegisterSlashCommand("vwai", "OnSlashCommand", self)
	Apollo.CreateTimer("WhereAmITimer", 0.250, true)
	Apollo.RegisterTimerHandler("WhereAmITimer", "OnTimer", self)

	self.wndMain = Apollo.LoadForm("WhereAmI.xml", "WhereAmIForm",  nil, self)
	self.wndMain:ToFront()

	self:SetSize()
	self:SetOpacity()
	self:SetTextAlign()
	self:OnTimer()

	Apollo.StartTimer("WhereAmITimer")
end

function WhereAmI:OnTimer()
	local unit = GameLib.GetPlayerUnit()
	if not unit then return end
	local tPos = unit:GetPosition()
	if tPos then
		local sHeight = ""
		if self.settings.bShowHeight then
			sHeight = ", " .. self:Round(tPos.y)
		end
		self.wndMain:FindChild("LocationText"):SetText(self:Round(tPos.x) .. ", " .. self:Round(tPos.z) .. sHeight)
	else
		self.wndMain:FindChild("LocationText"):SetText("I'm lost!")
	end
end

function WhereAmI:SetPosition()
	local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
	local nLeft, nTop, nRight, nBottom = self.wndMain:GetRect()
	local nWidth, nHeight = nRight - nLeft, nBottom - nTop
	if self.settings.tMainPosition[1] then
		self.wndMain:Move(self:Round(nScreenWidth * self.settings.tMainPosition[1]), self:Round(nScreenHeight * self.settings.tMainPosition[2]), nWidth, nHeight)
	else
		self.wndMain:Move(self:Round((nScreenWidth / 2) - (nWidth / 2)), self:Round((nScreenHeight / 2) - (nHeight / 2)), nWidth, nHeight)
	end
end

function WhereAmI:SetOpacity()
	self.wndMain:FindChild("Background"):SetOpacity(self.settings.nMainOpacity, 1000)
end

function WhereAmI:SetSize()
	if self.settings.bShowHeight then
		self.wndMain:SetAnchorOffsets(0, 0, 150, 32)
	else
		self.wndMain:SetAnchorOffsets(0, 0, 100, 32)
	end
	self:SetPosition()
end

function WhereAmI:SetTextAlign()
	self.wndMain:FindChild("LocationText"):SetTextFlags("DT_CENTER", false)
	self.wndMain:FindChild("LocationText"):SetTextFlags("DT_RIGHT", false)
	if self.settings.nTextAlign > 1 then
		self.wndMain:FindChild("LocationText"):SetTextFlags("DT_" .. arTextAligns[self.settings.nTextAlign]:upper(), true)
	end
end

function WhereAmI:Round(n)
	return math.floor(n + 0.5)
end

function WhereAmI:OnSlashCommand(strCmd, strParam)
	if strParam ~= "" then
		local tParams = {}
		for w in string.gmatch(strParam, "%S*") do
			if w ~= "" then
				if tonumber(w) then
					table.insert(tParams, tonumber(w))
				else
					table.insert(tParams, string.lower(w))
				end
			end
		end
		if tParams[1] == "reset" then
			self.settings.tMainPosition = {nil, nil}
			self:SetPosition()
			self:WriteToChat("Position reset.")
		elseif tParams[1] == "opacity" then
			if tParams[2] and tonumber(tParams[2]) then
				if tParams[2] >= 0 and tParams[2] <= 100 then
					self.settings.nMainOpacity = (math.floor(tParams[2] + 0.5)) / 100
					self:SetOpacity()
					self:WriteToChat("Opacity set to " .. math.floor(tParams[2] + 0.5) .. ".")
				else
					self:WriteToChat("Opacity must be between 0 and 100.")
				end
			else
				self:WriteToChat("You need to specify a number between 0 and 100.")
			end
		elseif tParams[1] == "height" then
			self.settings.bShowHeight = not self.settings.bShowHeight
			if self.settings.bShowHeight then
				self:WriteToChat("Now showing height.")
			else
				self:WriteToChat("Height is no longer shown.")
			end
			self:SetSize()
		elseif tParams[1] == "move" then
			self.bIsMoving = not self.bIsMoving
			self.wndMain:SetStyle("Moveable", self.bIsMoving)
			self.wndMain:SetStyle("IgnoreMouse", not self.bIsMoving)
			if self.bIsMoving then
				self:WriteToChat("You can now move Where Am I?")
			else
				self:WriteToChat("Where Am I? is now position locked.")
			end
		elseif tParams[1] == "align" then
			if tParams[2] then
				if tonumber(tParams[2]) then
					self.settings.nTextAlign = tParams[2]
					self:SetTextAlign()
				else
					local bFound = false
					for k, v in pairs(arTextAligns) do
						if v == tParams[2] then
							bFound = true
							self.settings.nTextAlign = k
							break
						end
					end
					if bFound then
						self:WriteToChat("Text alignment set to " .. arTextAligns[self.settings.nTextAlign] .. ".")
						self:SetTextAlign()
					else
						self:WriteToChat("Allowed align options are: \"left\", \"center\" & \"right\" (without quotation marks).")
					end
				end
			else
				self:WriteToChat("Please specify an align option. Allowed align options are: \"left\" or \"1\", \"center\" or \"2\" & \"right\" or \"3\" (without quotation marks).")
			end
		else
			self:ShowHelp()
		end
	else
		self:ShowHelp()
	end
end

function WhereAmI:ShowHelp()
	local sHeightState = "Off"
	if self.settings.bShowHeight then sHeightState = "On" end
	self:WriteToChat("Available commands for Where Am I?:")
	self:WriteToChat("\"move\", toggles the ability to move the window.")
	self:WriteToChat("\"height\" toggles showing height on/off. Currently: " .. sHeightState .. ".")
	self:WriteToChat("\"opacity x\" sets the opacity of the background. x needs to be a number between 0 and 100. Currently: " .. self.settings.nMainOpacity .. ".")
	self:WriteToChat("\"align x\" sets the text alignment. x needs to either the alignment or the matching number value as shown below.")
	self:WriteToChat("- Available align options: \"left\" or \"1\", \"center\" or \"2\" & \"right\" or \"3\" (without quotation marks).")
end

function WhereAmI:WriteToChat(text)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, text)
end

function WhereAmI:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Account then return nil end
	return self.settings
end

function WhereAmI:OnRestore(eLevel, tData)
	if tData then
		for k, v in pairs(tData) do
			self.settings[k] = v
		end
		if self.bCharacterCreated then
			self:SetSize()
			self:SetOpacity()
			self:SetTextAlign()
		end
	end
end

function WhereAmI:OnMainWindowMove(wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom)
	local nScreenWidth, nScreenHeight = Apollo.GetScreenSize()
	local nPosLeft, nPosTop = self.wndMain:GetPos()
	self.settings.tMainPosition[1], self.settings.tMainPosition[2] = nPosLeft / nScreenWidth, nPosTop / nScreenHeight
end


local WhereAmIInst = WhereAmI:new()
WhereAmIInst:Init()
